# -*- coding: utf-8 -*-
# Generated by Django 1.9.3.dev20160204092059 on 2016-02-18 08:32
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('raster', '0030_auto_20160111_0813'),
    ]

    operations = [
        migrations.AddField(
            model_name='rasterlayer',
            name='max_zoom',
            field=models.IntegerField(blank=True, help_text=b'Leave blank to automatically determine the max zoom levelfrom the raster scale. Otherwise the raster parsed up tothe zoom level specified here.', null=True),
        ),
        migrations.AddField(
            model_name='rasterlayer',
            name='srid',
            field=models.IntegerField(blank=True, help_text=b'Leave blank to use the internal raster srid. If a srid isspecified here, it will be used for all calculations.', null=True),
        ),
    ]
