# -*- coding: utf-8 -*-
# Generated by Django 1.9b1 on 2015-11-10 08:25
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('raster', '0022_auto_20151110_0810'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='rasterlayer',
            name='parse_log',
        ),
    ]
